#
# Utility functions for wealth distribution estimation after
#
# C. Dalitz: "Estimating Wealth Distribution: Top Tail and Inequality."
# Technischer Bericht Nr. 2016-01, Hochschule Niederrhein,
# Fachbereich Elektrotechnik und Informatik, 2016 
#
# Christoph Dalitz, version 1.0, 2016-02-11
#
# License: GNU GPL version 2
#


# numeric integration with trapezoidal rule
integrate.trapez <- function(x, y) {
  i <- 2:length(x)
  return (as.double( (x[i] - x[i-1]) %*% (y[i] + y[i-1])) / 2.0)
}

# LSQ fit to Pareto distribution
# returns (alpha, KS, CM)
lsq.pareto <- function(x, wmin, withintercept=FALSE) {
  w <- x[x$wealth >= wmin,]
  w <- w[order(w$wealth, decreasing=TRUE),]
  Nwmin <- sum(w$weight)
  Ni <- cumsum(w$weight)
  if (withintercept) {
    lsq <- lm(y ~ x, data=data.frame(x=-log(w$wealth/wmin), y= log(Ni/Nwmin)))
    alpha <- lsq$coefficients[2]
  } else {
    lsq <- lm(y ~ 0 + x, data=data.frame(x=-log(w$wealth/wmin), y= log(Ni/Nwmin)))
    alpha <- lsq$coefficients[1]
  }
  # Kolmogoroff, Smirnoff goodness of fit
  KS <- max(abs(Ni/Nwmin - (w$wealth/wmin)**(-alpha)))
  # Cramer, van Mieses goodness of fit
  x <- rev(w$wealth)
  y <- rev((Ni/Nwmin - (w$wealth/wmin)**(-alpha))**2 * alpha*(wmin**alpha) * (w$wealth)**(-alpha-1))
  CM <- integrate.trapez(x,y)
  return(data.frame(alpha=alpha, KS=KS, CM=CM))
}

# ML fit to Pareto distribution
# returns (alpha, KS, CM)
ml.pareto <- function(x, wmin) {
  w <- x[x$wealth >= wmin,]
  w <- w[order(w$wealth, decreasing=TRUE),]
  Nwmin <- sum(w$weight)
  Ni <- cumsum(w$weight)
  alpha <- 1 / (sum( log(w$wealth/wmin) * w$weight / Nwmin ))
  # Kolmogoroff, Smirnoff goodness of fit
  KS <- max(abs(Ni/Nwmin - (w$wealth/wmin)**(-alpha)))
  # Cramer, van Mieses goodness of fit with trapezoidal integration
  x <- rev(w$wealth)
  y <- rev((Ni/Nwmin - (w$wealth/wmin)**(-alpha))**2 * alpha*(wmin**alpha) * (w$wealth)**(-alpha-1))
  CM <- integrate.trapez(x,y)
  return(data.frame(alpha=alpha, KS=KS, CM=CM))
}

# alpha estimator with van der Wijk's law to Pareto distribution
# returns (alpha, KS, CM)
wijk.pareto <- function(x, wmin) {
  w <- x[x$wealth >= wmin,]
  w <- w[order(w$wealth, decreasing=TRUE),]
  Nwmin <- sum(w$weight)
  Ni <- cumsum(w$weight)
  alpha <- 1 / (1 - wmin*sum(w$weight) / (w$wealth %*% w$weight))
  # Kolmogoroff, Smirnoff goodness of fit
  KS <- max(abs(Ni/Nwmin - (w$wealth/wmin)**(-alpha)))
  # Cramer, van Mieses goodness of fit with trapezoidal integration
  x <- rev(w$wealth)
  y <- rev((Ni/Nwmin - (w$wealth/wmin)**(-alpha))**2 * alpha*(wmin**alpha) * (w$wealth)**(-alpha-1))
  CM <- integrate.trapez(x,y)
  return(data.frame(alpha=alpha, KS=KS, CM=CM))
}


# compute the wealth share of the top p percent
# x must have columns wealth and weight with weight normalized
# such that the total density is normalized to one
compute.share <- function(p, x, w0, C, alpha, wmax=NA)
{
  x <- x[order(x$wealth, decreasing=TRUE),]
  if (is.na(wmax)) {
    cutoff <- 0.0
  } else {
    cutoff <- wmax**(1-alpha)
  }
  # compute wealth threshold for top p percent
  wp <- (C/(alpha*p))**(1/alpha)
  lessw0 <- x[x$wealth < w0,]
  denominator <- as.numeric(lessw0$wealth %*% lessw0$weight) +
    C*(w0**(1-alpha) - cutoff) / (alpha-1)
  if (wp >= w0) {
    numerator <- C * (wp**(1-alpha) - cutoff) / (alpha-1)
  } else {
    # interpolate linearly to find value that exactly surpasses percent
    sumweights <- cumsum(x$weight)
    i <- min(which(sumweights >= p))
    wp <- (p - sumweights[i-1])*(x$wealth[i]-x$wealth[i-1]) / (sumweights[i]-sumweights[i-1]) + x$wealth[i-1]
    #cat(sprintf("x[i]=%f (%f)\n", x[i,]$wealth, sumweights[i]))
    #print(wp)
    wpw0 <- x[x$wealth >= wp & x$wealth < w0,]
    numerator <- as.numeric(wpw0$wealth %*% wpw0$weight) +
      C * (w0**(1-alpha) - cutoff) / (alpha-1)
  }
  return(numerator / denominator)
}

#
# compute Pareto normalization constant C after Bach et al. (2015)
#
C.bach <-  function(x, alpha, wmin, w0, wmax) {
  if (is.na(wmax)) {
    C <- alpha * w0**alpha * sum(x[x$wealth >= w0, "weight"]) /
      sum(x$weight)
  } else {
    C <- alpha * sum(x[x$wealth >= w0, "weight"]) /
      (sum(x$weight) * (w0**(-alpha) - wmax**(-alpha)))
  }
  x.bach <- data.frame(wealth=x$wealth, weight=x$weight)
  x.bach$weight <- x$weight / sum(x$weight)
  return(list(C=C, x=x.bach))
}


#
# compute Pareto normalization constant C after Eckerstorfer et al. (2015)
#
C.eck <-  function(x, alpha, wmin, w0, wmax) {
  if (is.na(wmax)) {
    C <- alpha * wmin**alpha * sum(x[x$wealth >= wmin, "weight"]) /
      sum(x$weight)
  } else {
    C <- alpha * sum(x[x$wealth >= wmin, "weight"]) /
      (sum(x$weight) * (wmin**(-alpha) - wmax**(-alpha)))
  }
  beta <- ( sum(x$weight) -
           sum(x[x$wealth > wmin & x$wealth <= w0, "weight"]) -
           (wmin/w0)**alpha * sum(x[x$wealth > wmin, "weight"]) ) /
             sum(x[x$wealth <= wmin, "weight"])
  x.eck <- data.frame(wealth=x$wealth, weight=x$weight)
  x.eck[x.eck$wealth <= wmin, "weight"] <- beta * x.eck[x.eck$wealth <= wmin, "weight"] / sum(x$weight)
  x.eck[x.eck$wealth > wmin, "weight"] <- x.eck[x.eck$wealth > wmin, "weight"] / sum(x$weight)  
  return(list(C=C, x=x.eck))
}

#
# compute Pareto normalization constant C after Dalitz (2016)
#
C.dalitz <-  function(x, alpha, w1, w0, wmax, richlist) {
  richlist <- richlist[order(richlist$wealth, decreasing=TRUE),]
  Nw1 <- sum(richlist[richlist$wealth >= w1,]$weight)
  w1 <- richlist[min(which(richlist$wealth < w1)), "wealth"]
  #cat(sprintf("w1 reset to %f\n", w1))
  if (is.na(wmax)) {
    N <- sum(x[x$weight <= w0, "weight"]) + Nw1*(w1/w0)**alpha
    C <- alpha * w1**alpha * Nw1 / N
  } else {
    N <- sum(x[x$weight <= w0, "weight"]) +
      Nw1 * (w0**(-alpha) - wmax**(-alpha)) / (w1**(-alpha) - wmax**(-alpha))
    C <- alpha * Nw1 / (N * (w1**(-alpha) - wmax**(-alpha)))
  }
  x.dalitz <- data.frame(wealth=x$wealth, weight=x$weight)
  x.dalitz$weight <- x$weight / N
  return(list(C=C, x=x.dalitz))
}
