#
# Plot goodness-of-fit criteria KS and CM as a function of w_min after
#
# C. Dalitz: "Estimating Wealth Distribution: Top Tail and Inequality."
# Technischer Bericht Nr. 2016-01, Hochschule Niederrhein,
# Fachbereich Elektrotechnik und Informatik, 2016 
#
# Christoph Dalitz, version 1.0, 2016-02-11
#
# License: GNU GPL version 2
#

source("functions.r")

# set to TRUE when optimal w_min shall be found for LSQ and Wijk too
other.too <- TRUE

x <- read.csv("data.csv")
x <- aggregate(list(weight = x$weight), by=list(wealth=x$wealth), FUN=sum)
x <- x[order(x$wealth, decreasing=FALSE),]

# wmin values
xmins <- sort(unique(x$wealth[0.5*10**6 <= x$wealth & x$wealth <= 0.7*10**6]))
cmfactor <- 250
n <- length(xmins)


# Optimum for ML
result <- data.frame()
for (xm in xmins) {
  result <- rbind(result, ml.pareto(x, xm))
}
par(lwd=3, ps=24, mar=c(5,3,1,1)+0.1, family="Times")
plot(xmins/10^6, result$KS, main=NA, ylab=NA, , cex.lab=1.0, xlab=expression(paste(italic('w'[min]), '  [Mio Euro]')), type="l", col="red2")
lines(xmins/10^6, cmfactor*result$CM, col="black", lty=1)
legend("topright", c(expression(italic("KS")), bquote(.(cmfactor)%.%italic("CM"))), col=c("red2", "black"), lty=c(1,1), bg='white', y.intersp=1.5)
rKS <- ml.pareto(x, xmins[which.min(result$KS)])
rCM <- ml.pareto(x, xmins[which.min(result$CM)])
cat(sprintf("ML-Fit:\n    alpha=%f, wmin=%.0f, KS=%f\n", rKS$alpha, xmins[which.min(result$KS)], rKS$KS))
cat(sprintf("    alpha=%f, wmin=%.0f, CM=%f\n", rCM$alpha, xmins[which.min(result$CM)], rCM$CM))

if (other.too) {
  # Optimum for LSQ with intercept
  result <- data.frame()
  for (xm in xmins) {
    result <- rbind(result, lsq.pareto(x, xm, TRUE))
  }
  #plot(xmins, result$KS, type="l", col="red2")
  #lines(xmins, 150*result$CM, col="black")
  #legend("topright", c("KS", "CM"), col=c("red2", "black"), lty=c(1,1))
  rKS <- lsq.pareto(x, xmins[which.min(result$KS)], TRUE)
  rCM <- lsq.pareto(x, xmins[which.min(result$CM)], TRUE)
  cat(sprintf("LSQ-Fit (with intercept):\n    alpha=%f, wmin=%.0f, KS=%f\n", rKS$alpha, xmins[which.min(result$KS)], rKS$KS))
  cat(sprintf("    alpha=%f, wmin=%.0f, CM=%f\n", rCM$alpha, xmins[which.min(result$CM)], rCM$CM))

  # Optimum for LSQ without intercept
  result <- data.frame()
  for (xm in xmins) {
    result <- rbind(result, lsq.pareto(x, xm))
  }
  #plot(xmins, result$KS, type="l", col="red2", xlab="w_min", ylab="goodness-of-fit")
  #lines(xmins, 100*result$CM, col="black")
  #legend("bottomright", c("KS", "100*CM"), col=c("red2", "black"), lty=c(1,1))
  rKS <- lsq.pareto(x, xmins[which.min(result$KS)])
  rCM <- lsq.pareto(x, xmins[which.min(result$CM)])
  cat(sprintf("LSQ-Fit (no intercept):\n    alpha=%f, wmin=%.0f, KS=%f\n", rKS$alpha, xmins[which.min(result$KS)], rKS$KS))
  cat(sprintf("    alpha=%f, wmin=%.0f, CM=%f\n", rCM$alpha, xmins[which.min(result$CM)], rCM$CM))

  # Optimum for Wijk without intercept
  result <- data.frame()
  for (xm in xmins) {
    result <- rbind(result, wijk.pareto(x, xm))
  }
  #plot(xmins, result$KS, type="l", col="red2", xlab="w_min", ylab="goodness-of-fit")
  #lines(xmins, 100*result$CM, col="black")
  #legend("bottomright", c("KS", "100*CM"), col=c("red2", "black"), lty=c(1,1))
  rKS <- wijk.pareto(x, xmins[which.min(result$KS)])
  rCM <- wijk.pareto(x, xmins[which.min(result$CM)])
  cat(sprintf("Wijk-Fit:\n    alpha=%f, wmin=%.0f, KS=%f\n", rKS$alpha, xmins[which.min(result$KS)], rKS$KS))
  cat(sprintf("    alpha=%f, wmin=%.0f, CM=%f\n", rCM$alpha, xmins[which.min(result$CM)], rCM$CM))
}
