#
# Monte-Carlo simulation for Pareto parameter estimation after
#
# C. Dalitz: "Estimating Wealth Distribution: Top Tail and Inequality."
# Technischer Bericht Nr. 2016-01, Hochschule Niederrhein,
# Fachbereich Elektrotechnik und Informatik, 2016 
#
# Christoph Dalitz, version 1.0, 2016-02-11
#
# License: GNU GPL version 2
#

# for Pareto random generator
library(actuar)

source("functions.r")

wmin <- 500000
wmax <- 75*10**6
alpha <- 1.5
N.total <- 2.5*10**6
N.sample <- 5000
N.rep <- 1000

# Simulation 1:
# no cutoff, no weighting
#----------------------------------
cat("1) no cutoff, no weighting\n")
res.ml <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq2 <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.wijk <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
for (i in 1:N.rep) {
  wi <- rpareto(N.sample, alpha, wmin) + wmin
  ni <- rep(1, length(wi))
  x <- data.frame(wealth=wi, weight=ni)
  res.ml[i,] <- ml.pareto(x, wmin)
  res.lsq[i,] <- lsq.pareto(x, wmin)
  res.lsq2[i,] <- lsq.pareto(x, wmin, TRUE)
  res.wijk[i,] <- wijk.pareto(x, wmin)
}
cat(sprintf("ML:   alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.ml$alpha), sd(res.ml$alpha), var(res.ml$alpha) + mean(alpha-res.ml$alpha)**2, mean(res.ml$KS), mean(res.ml$CM)))
cat(sprintf("LSQ:  alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq$alpha), sd(res.lsq$alpha), var(res.lsq$alpha) + mean(alpha-res.lsq$alpha)**2, mean(res.lsq$KS), mean(res.lsq$CM)))
cat(sprintf("LSQ2: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq2$alpha), sd(res.lsq2$alpha), var(res.lsq2$alpha) + mean(alpha-res.lsq2$alpha)**2, mean(res.lsq2$KS), mean(res.lsq2$CM)))
cat(sprintf("Wijk: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.wijk$alpha), sd(res.wijk$alpha), var(res.wijk$alpha) + mean(alpha-res.wijk$alpha)**2, mean(res.wijk$KS), mean(res.wijk$CM)))

# Simulation 2:
# with cutoff, no weighting
#----------------------------------
cat("2) with cutoff, no weighting\n")
res.ml <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq2 <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.wijk <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
for (i in 1:N.rep) {
  wi <- rpareto(N.sample*2, alpha, wmin) + wmin
  wi <- wi[wi < wmax]
  wi <- head(wi, n=N.sample)
  ni <- rep(1, length(wi))
  x <- data.frame(wealth=wi, weight=ni)
  res.ml[i,] <- ml.pareto(x, wmin)
  res.lsq[i,] <- lsq.pareto(x, wmin)
  res.lsq2[i,] <- lsq.pareto(x, wmin, TRUE)
  res.wijk[i,] <- wijk.pareto(x, wmin)
}
cat(sprintf("ML:   alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.ml$alpha), sd(res.ml$alpha), var(res.ml$alpha) + mean(alpha-res.ml$alpha)**2, mean(res.ml$KS), mean(res.ml$CM)))
cat(sprintf("LSQ:  alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq$alpha), sd(res.lsq$alpha), var(res.lsq$alpha) + mean(alpha-res.lsq$alpha)**2, mean(res.lsq$KS), mean(res.lsq$CM)))
cat(sprintf("LSQ2: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq2$alpha), sd(res.lsq2$alpha), var(res.lsq2$alpha) + mean(alpha-res.lsq2$alpha)**2, mean(res.lsq2$KS), mean(res.lsq2$CM)))
cat(sprintf("Wijk: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.wijk$alpha), sd(res.wijk$alpha), var(res.wijk$alpha) + mean(alpha-res.wijk$alpha)**2, mean(res.wijk$KS), mean(res.wijk$CM)))

# Simulation 3:
# no cutoff, with weighting
#----------------------------------
cat("3) no cutoff, with weighting\n")
res.ml <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq2 <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.wijk <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
for (i in 1:N.rep) {
  wi <- rpareto(N.sample, alpha, wmin) + wmin
  wi <- wi[order(wi, decreasing=FALSE)]
  j <- 2:N.sample
  wmid <- c(wmin, (wi[j-1]+ wi[j])/2)
  ni <- (wmid[j-1]/wmin)**(-alpha) - (wmid[j]/wmin)**(-alpha)
  ni <- c(ni, (wmid[N.sample]/wmin)**(-alpha))
  ni <- floor(ni*N.total)
  x <- data.frame(wealth=wi, weight=ni)
  res.ml[i,] <- ml.pareto(x, wmin)
  res.lsq[i,] <- lsq.pareto(x, wmin)
  res.lsq2[i,] <- lsq.pareto(x, wmin, TRUE)
  res.wijk[i,] <- wijk.pareto(x, wmin)
}
cat(sprintf("ML:   alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.ml$alpha), sd(res.ml$alpha), var(res.ml$alpha) + mean(alpha-res.ml$alpha)**2, mean(res.ml$KS), mean(res.ml$CM)))
cat(sprintf("LSQ:  alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq$alpha), sd(res.lsq$alpha), var(res.lsq$alpha) + mean(alpha-res.lsq$alpha)**2, mean(res.lsq$KS), mean(res.lsq$CM)))
cat(sprintf("LSQ2: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq2$alpha), sd(res.lsq2$alpha), var(res.lsq2$alpha) + mean(alpha-res.lsq2$alpha)**2, mean(res.lsq2$KS), mean(res.lsq2$CM)))
cat(sprintf("Wijk: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.wijk$alpha), sd(res.wijk$alpha), var(res.wijk$alpha) + mean(alpha-res.wijk$alpha)**2, mean(res.wijk$KS), mean(res.wijk$CM)))

# Simulation 4:
# with cutoff, with weighting
#----------------------------------
cat("4) with cutoff, with weighting\n")
res.ml <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.lsq2 <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
res.wijk <- data.frame(alpha=numeric(N.rep), KS=numeric(N.rep), CM=numeric(N.rep))
for (i in 1:N.rep) {
  wi <- rpareto(N.sample*2, alpha, wmin) + wmin
  wi <- wi[wi < wmax]
  wi <- head(wi, n=N.sample)
  wi <- wi[order(wi, decreasing=FALSE)]
  j <- 2:N.sample
  wmid <- c(wmin, (wi[j-1]+ wi[j])/2, wmax)
  j <- 2:(N.sample+1)
  ni <- (wmid[j-1]/wmin)**(-alpha) - (wmid[j]/wmin)**(-alpha)
  ni <- floor(ni*N.total)
  x <- data.frame(wealth=wi, weight=ni)
  res.ml[i,] <- ml.pareto(x, wmin)
  res.lsq[i,] <- lsq.pareto(x, wmin)
  res.lsq2[i,] <- lsq.pareto(x, wmin, TRUE)
  res.wijk[i,] <- wijk.pareto(x, wmin)
}
cat(sprintf("ML:   alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.ml$alpha), sd(res.ml$alpha), var(res.ml$alpha) + mean(alpha-res.ml$alpha)**2, mean(res.ml$KS), mean(res.ml$CM)))
cat(sprintf("LSQ:  alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq$alpha), sd(res.lsq$alpha), var(res.lsq$alpha) + mean(alpha-res.lsq$alpha)**2, mean(res.lsq$KS), mean(res.lsq$CM)))
cat(sprintf("LSQ2: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.lsq2$alpha), sd(res.lsq2$alpha), var(res.lsq2$alpha) + mean(alpha-res.lsq2$alpha)**2, mean(res.lsq2$KS), mean(res.lsq2$CM)))
cat(sprintf("Wijk: alpha=%.4f (sd=%.4f), MSE=%f, KS=%f, CM=%f\n", mean(res.wijk$alpha), sd(res.wijk$alpha), var(res.wijk$alpha) + mean(alpha-res.wijk$alpha)**2, mean(res.wijk$KS), mean(res.wijk$CM)))

